*============================================================================
* mcmcbgr:  plot convergence of serveral chains
* Author:  John Thompson
* Date:    Dec 2012
*============================================================================

program mcmcbgr , sortpreserve
   version 12.1
   syntax varlist (max = 1 min = 1) [if] [in] , ///
   Chain(varname) [       ///  chain identifier
   GOPTions(string asis)  ///  options for the separate graphs
   CGOPTions(string asis) ///  options for combining the two graphs
   Iteration(varname)     ///  variable giving the iteration number
   M(integer 20)          ///  number of plotting points
   BYChain                ///  show each chain rather than their average
   Variance               ///  use variance rather than 80% limit
   Save(string)           ///  file for saving the plotting points
   ]
   
*---------------------------------
* Check options
*---------------------------------
   if `m' < 1 {
      di as err "mcmcintervals: option m must be positive"
      exit(499)
   }
*---------------------------------
* Create plot
*---------------------------------
   quietly {
      preserve
      marksample touse
*---------------------------------
* count number of chains
*---------------------------------
      qui levelsof `chain' if `touse', local(clist)
      local nc : word count `clist'
*-----------------------------
* parse the save option
*-----------------------------      
      if "`save'" != "" {
         tokenize `"`save'"',parse(",")
         local save = trim(`"`1'"')
         local dreplace `"`3'"'
      }
*---------------------------------
* create x-axis variable
*---------------------------------
      tempvar t h
      if "`iteration'" == "" {
         gen `h' = _n
         sort `chain' `h'
         by `chain': egen `t' = seq()
         local orderlab = "Iteration"
      }
      else {
         gen `t' = `iteration'
         local orderlab : variable label `iteration'
         if "`orderlab'" == "" local orderlab "`iteration'"
      }
      keep if `touse'
      su `t'
      local mint = r(min)
      local maxlength = 0
      foreach c of local clist {
         su `t' if `chain' == `c'
         local maxlength = max(`maxlength',r(max))
      }
      sort `chain' `t'
      local binsize = `maxlength'/`m'
      local nbins = `m'
*---------------------------------
* calculate the plotting points
*---------------------------------
      tempvar I80 limit aI80 R dfm dfv df
      gen `I80' = .
      gen `limit' = .
      gen `dfm' = .
      gen `dfv' = .
      gen `df' = .
      
      local list ""
      local plot ""
      forvalues i = 1 / `nc' {
         tempvar I80_`i'
         gen `I80_`i'' = .
         local list "`list' `I80_`i''"
         local plot "`plot' (line `I80_`i'' `limit' , lpat(dash) )"
      }
      forvalues i = 1 / `nbins' {
         local upper = `i'*`binsize' + `mint'-1
         local half = `i'*`binsize'/2 + `mint'
         replace `limit' = `upper' in `i'
*---------------------------------
* 80% intervals
*---------------------------------
         if "`variance'" == "" {
            centile `varlist' if `t' < = `upper' & `t' > `half' , centile(10 90)
            replace `I80' = r(c_2)-r(c_1) in `i'
            forvalues j = 1 / `nc' {
               centile `varlist' if `t' < = `upper' & `t' > `half' & `chain' == `j' , centile(10 90)
               replace `I80_`j'' = r(c_2)-r(c_1) in `i'
            }
         }
*---------------------------------
* variances
*---------------------------------
         else {
            su `varlist' if `t' < = `upper' & `t' > `half'
            local m = r(mean)
            local ss = 0
            local sv = 0
            forvalues j = 1 / `nc' {
               su `varlist' if `t' < = `upper' & `t' > `half' & `chain' == `j'
               replace `dfm' = r(mean) in `j'
               replace `dfv' = r(Var) in `j'
               replace `I80_`j'' = r(Var) in `i'
               local sv = `sv' + r(Var)
               local ss = `ss' + (r(mean)-`m')^2
               local n = r(N)
            }
            local W = `sv'/`nc'
            local B = `n'*`ss'/(`nc'-1)
            local splus = (`n'-1)*`W'/`n' + `B'/`n'
            local V = `splus' + `B'/(`n'*`nc')
            replace `I80' = `V' in `i'
            su `dfv'
            local v1 = r(Var)
            local vv = ((`n'-1)/`n')^2*r(Var)/`nc'+((`nc'+1)/(`nc'*`n'))^2*2*`B'*`B'/(`nc'-1)
            su `dfm'
            local gm = r(mean)
            local v2 = r(Var)
            corr `dfv' `dfm'
            local cv1 = r(rho)*sqrt(`v1'*`v2')
            replace `dfm' = `dfm'*`dfm'
            su `dfm'
            local v3 = r(Var)
            corr `dfv' `dfm'
            local cv2 = r(rho)*sqrt(`v1'*`v3')
            local vv = `vv' + (2*(`nc'+1)*(`n'-1)/(`nc'*`n'*`n'))*(`n'/`nc')*(`cv2'-2*`gm'*`cv1')
            local edf = 2*`V'*`V'/`vv'
            replace `df' = `edf' in `i'
         }
      }
*---------------------------------
* Calculate R statistic
*---------------------------------
      egen `aI80' = rmean(`list')
      if "`variance'" == "" {
         local Ylabel = "80% Intervals"
         local Rlabel = "R"
         gen `R' = `I80' / `aI80'
      }
      else {
         local Ylabel = "Variance Estimates"
         local Rlabel = "R"
         gen `R' = ((`df'+3)/(`df'+1))*`I80' / `aI80'
      }
*---------------------------------
* Create upper and lower plots
*---------------------------------
      tempfile p1 p2
      line `R' `limit' , lcol(black) yline(1, lpa(dash) lcol(dkblue)) ytitle("`Rlabel'") xtitle("`orderlab'") ///
      saving(`p1'.gph , replace) `goptions' ylabel(,angle(0)) nodraw
      
      if "`bychain'" != "" {
         twoway (line `I80' `limit', lcol(black) ) `plot' , legend(off) ///
         ytitle("`Ylabel'") xtitle("`orderlab'") ylabel(,angle(0)) ///
         saving(`p2'.gph , replace) `goptions' nodraw
      }
      else {
         twoway (line `I80' `limit', lcol(black) ) (line `aI80' `limit' , lpa(dash) lcol(dknavy)) ,  ///
         ytitle("`Ylabel'") xtitle("`orderlab'") ylabel(,angle(0)) ///
         legend( label(1 "Pooled") label(2 "Within") pos(1) ring(0) col(1)) ///
         saving(`p2'.gph , replace) `goptions' nodraw 
      }
*---------------------------------
* combine the plots
*---------------------------------
      graph combine `p1'.gph `p2'.gph , row(2)   `cgoptions'
*---------------------------------
* save plotting points if needed
*---------------------------------
      if `"`save'"' != "" {
         keep `limit' `R' `df' `I80' `aI80' `list'
         drop if `I80' == .
         rename `limit' n
         rename `R' R
         rename `I80' pooled
         rename `aI80' average
         rename `df' df
         drop if n == .
         if "`variance'" == "" {
            drop df
         }
         local i = 0
         foreach v of local list {
            local i = `i' + 1
            rename `v' chain`i'
         }
         qui save `"`save'"' , `dreplace'
      }
      restore
   }
end
