*============================================================================
* mcmccusum:  Cusum plots of an MCMC analysis
* Author:    John Thompson
* Date:      Dec 2012
*============================================================================

program mcmccusum, sortpreserve
   version 12.1
   syntax varlist [if] [in] , [ ///
   GOPTions(string asis)        ///  options for graph command
   CGOPTions(string asis)       ///  options for graph combine command
   Chain(varname)               ///  variable containing the chain number (only when multiple chains are run)
   Iteration(varname)           ///  variable containing the variable for the x-axis
   OVERlay                      ///  plot graphs from different chains on the same axes
   Reference(integer 0)         ///  number of reference cusum plots for comparison
   ]
   
   marksample touse
   
*---------------------------------
* Check options
*---------------------------------
   if `reference' < 0 {
      di as err "warning: reference must be positive - treated as if zero"
      local reference = 0
   }
   if "`chain'" == "" & "`overlay'" != "" {
      di as err "warning: overlay ignored - option only works with multiple chains"
      local overlay ""
   }
   if `reference' != 0 & "`overlay'" != "" {
      di as err "warning: reference ignored - option not available for overlayed plots"
      local reference = 0
   }
*---------------------------------
* Create plot
*---------------------------------
   quietly {
      preserve
*---------------------------------
* count number of parameters
*---------------------------------
      local np : word count `varlist'
*---------------------------------
* count number of chains
*---------------------------------
      if "`chain'" == "" {
         local nc = 1
      }
      else {
         levelsof `chain' if `touse', local(clist)
         local nc : word count `clist'
      }
      if "`overlay'" == "" {
         local np = `np'*`nc'
      }
      if `np' > 1 local nodraw "nodraw"
*---------------------------------
* create x-axis variable
*---------------------------------
      tempvar t h
      if "`iteration'" == "" {
         if "`chain'" == "" gen `t' = _n
         else {
            gen `h' = _n
            sort `chain' `h'
            by `chain': egen `t' = seq()
         }
         local orderlab = "Iteration"
      }
      else {
         gen `t' = `iteration'
         local orderlab : variable label `iteration'
         if "`orderlab'" == "" local orderlab "`iteration'"
      }
      keep if `touse'
      sort `chain' `t'
*---------------------------------
* interval size for each plot
*---------------------------------
      if "`chain'" != "" {
         local n = 0
         local nm = 1E10
         foreach c of local clist {
            su `t' if `chain' == `c' 
            local n = max(`n',r(max))
            local nm = min(`nm',r(min))
         }
      }
      else {
         su `t' 
         local n = r(max)
         local nm = r(min)
      }
      local nm1 = `nm' - 1
      local scale = int((`n'-`nm1')/5)
*---------------------------------
* plot each series
*---------------------------------
      local i = 0
      local cs = ""
      foreach v of varlist `varlist' {
*---------------------------------
* Single chain
*---------------------------------
         if "`chain'" == "" {
            local i = `i' + 1
            tempfile t`i'
            local s = "saving(`t`i''.gph,replace)"
            local cs "`cs' `t`i''.gph"
*---------------------------------
* find sum deviations
*---------------------------------
            su `v' if `touse'
            local mn = r(mean)
            local sd = r(sd)
            tempvar `v'_plus
            gen ``v'_plus' = `v'[_n-1]
            corr `v' ``v'_plus' if `touse'
            local rho = r(rho)
            local pref ""
            if `reference' != 0 {
               forvalues r = 1/`reference' {
                  local nvr = _N
                  tempvar sy`v'_`r' dsy`v'_`r'
                  local yv = `mn'
                  gen `sy`v'_`r'' = `yv'
                  forvalues ir = 2/`nvr' {
                     local yv = `rho'*`yv' + `mn'*(1-`rho') + sqrt(1-`rho'*`rho')*`sd'*rnormal()
                     replace `sy`v'_`r'' = `yv' in `ir'
                  }
                  
                  su `sy`v'_`r'' if `touse'
                  gen `dsy`v'_`r'' = sum(`sy`v'_`r''-r(mean)) if `touse'
                  local pref "`pref' (line `dsy`v'_`r'' `t' if `touse', lpat(dot) lcol(black))"
               }
            }
            tempvar dt`v'
            gen `dt`v'' = sum(`v'-`mn') if `touse'
*---------------------------------
* plot everything
*---------------------------------
            local yt "cusum(`v')"
            twoway (line `dt`v'' `t' if `touse', lpat(solid) lcol(dknavy)) `pref' , ///
            xtitle("`orderlab'") yline(0, lcol(black)) ///
            xscale(range(`nm1' `n')) xlabel(`nm1'(`scale')`n') ///
            ytitle("`yt'")  `s' legend(off) `nodraw' `goptions' 
         }
         else {
*---------------------------------
* Several chains - overlay
*---------------------------------
            if "`overlay'" != "" {
               local i = `i' + 1
               tempfile t`i'
               local s = "saving(`t`i''.gph,replace)"
               local cs "`cs' `t`i''.gph"
*---------------------------------
* find sum deviations
*---------------------------------
               local yt "cusum(`v')"
               local pc ""
               foreach c of local clist {
                  su `v' if `touse' & `chain' == `c'
                  local mn = r(mean)
                  tempvar dt`v'`c'
                  gen `dt`v'`c'' = sum(`v'-`mn') if `touse' & `chain' == `c'
                  local pc "`pc' (line `dt`v'`c'' `t' if `touse' & `chain' == `c', lpat(solid))"
               }
*---------------------------------
* plot everything
*---------------------------------
               twoway `pc' , xtitle("`orderlab'") yline(0, lcol(black)) ///
               xscale(range(`nm1' `n')) xlabel(`nm1'(`scale')`n') ///
               ytitle("`yt'")  legend(off) `s' `nodraw' `goptions'
            }
*---------------------------------
* Several chains - not overlayed
*---------------------------------
            else {
               tempvar `v'_plus order
               gen ``v'_plus' = `v'[_n-1]
               gen `order' = _n
               foreach c of local clist {
*---------------------------------
* find sum deviations
*---------------------------------
                  su `v' if `touse' & `chain' == `c'
                  local mn = r(mean)
                  local sd = r(sd)
                  count if `chain' == `c'
                  local nvr = r(N)
                  su `t' if `chain' == `c'
                  local mt = r(min)
                  su `order' if `t' == `mt' & `chain' == `c'
                  local base = r(sum)
                  corr `v' ``v'_plus' if `touse' & `chain' == `c'
                  local rho = r(rho)
                  local pref ""
                  if `reference' != 0 {
                     forvalues r = 1/`reference' {
                        tempvar dt`v' sy`v'_`r' dsy`v'_`r'
                        local yv = `mn'
                        gen `sy`v'_`r'' = `yv'
                        forvalues ir = 2/`nvr' {
                           local yv = `rho'*`yv' + `mn'*(1-`rho') + sqrt(1-`rho'*`rho')*`sd'*rnormal()
                           local p = `base' + `ir' - 1
                           replace `sy`v'_`r'' = `yv' in `p'
                        }
                        su `sy`v'_`r'' if `touse' & `chain' == `c'
                        gen `dsy`v'_`r'' = sum(`sy`v'_`r''-r(mean)) if `touse' & `chain' == `c'
                        local pref "`pref' (line `dsy`v'_`r'' `t' if `touse' & `chain' == `c', lpat(dot) lcol(black))"
                     }
                  }
                  tempvar dt`v'`c'
                  gen `dt`v'`c'' = sum(`v'-`mn') if `touse' & `chain' == `c'
                  local yt "cusum(`v')"
                  local i = `i' + 1
                  tempfile t`i'
                  local s = "saving(`t`i''.gph,replace)"
                  local cs "`cs' `t`i''.gph"
*---------------------------------
* plot everything
*---------------------------------
                  twoway (line `dt`v'`c'' `t' if `touse' & `chain' == `c', lpat(solid) lcol(dknavy)) `pref' , ///
                  yline(0, lcol(black)) ///
                  xscale(range(`nm1' `n')) xlabel(`nm1'(`scale')`n') ///
                  ytitle("`yt'") xtitle("`orderlab'(chain `c')")  `s' legend(off) `nodraw' `goptions'
               }
            }
         }
      }
*---------------------------------
* combine plots if needed
*---------------------------------
      if `np' > 1 {
         graph combine `cs' , `cgoptions'
      }
    restore
   }
end
