*============================================================================
* MC_DENSITY:  Estimated posterior density based on an MCMC run using Stata's kdensity command
* Author:      John Thompson
* Date:        Dec 2012
*============================================================================

program mcmcdensity, rclass
   version 12.1
   syntax varlist [if] [in] , [ ///
   Lbounds(string)              /// lboundser limit of density range
   Ubounds(string)              /// upper limit of density range
   KOPTions(string asis)        /// options passed to kdensity
   GOPTions(string asis)        /// options passed to graph twoway
   CGOPTions(string asis)       /// options passed to graph combine
   Save(string)                 /// .dta file for saving the plotting points
   ADDplot(string)              /// second plot added to the density
   ]
   
   quietly {
      marksample touse
*---------------------------------
* count number of parameters
*---------------------------------
      local np : word count `varlist'
      if `np' > 1 local nodraw "nodraw"
*---------------------------------
* set upper & lboundser limits
*---------------------------------
      tokenize "`lbounds'"
      forvalues j = 1/`np' {
         if "``j''" != "" local lbounds`j' "``j''"
         else local lbounds`j' "."
      }
      tokenize "`ubounds'"
      forvalues j = 1/`np' {
         if "``j''" != "" local ubounds`j' "``j''"
         else local ubounds`j' "."
      }
*---------------------------------
* details of save file
*---------------------------------
      tokenize `"`save'"',parse(",")
      local save = trim(subinstr(`"`1'"',`"""',"",.))
      local dreplace `"`3'"'
*---------------------------------
* process each variable
*---------------------------------
      tempvar y x f id my
      local cs ""
      local j = 0
      foreach v of varlist `varlist' {
         local ++j
         local vlab : var lab `v'
         if "`vlab'" == "" {
            local vlab "`v'"
         }
         preserve
         keep if `touse'
         gen `y' = `v'
*---------------------------------
* if bounded - reflect
*---------------------------------
         local set = 1
         gen `my' = 1
         local stacklist ""
         if "`lbounds`j''" != "." {
            tempvar ylb`j' mlb`j'
            gen `ylb`j'' = `lbounds`j'' - (`y'-`lbounds`j'')
            local set = `set' + 1
            gen `mlb`j'' = 0
            local stacklist "`mlb`j'' `ylb`j''"
         }
         if "`ubounds`j''" != "."{
            tempvar yub`j' mub`j'
            gen `yub`j'' = `ubounds`j'' + (`ubounds`j''-`y')
            local set = `set' + 1
            gen `mub`j'' = 0
            local stacklist "`stacklist' `mub`j'' `yub`j''"
         }
         if `set' > 1 {
            stack `my' `y' `stacklist' , into(`my' `y') clear
            su `y' if `my' == 1
            local bw = 100*r(sd)/(r(N)^0.8)
            local bw = "bw(`bw')"
         }
         local n = min(_N,250)
         kdensity `y' , gen(`x' `f') nograph n(`n') `bw' `koptions'
         if "`lbounds`j''" != "." {
            cap drop if `x' < `lbounds`j''
         }
         if "`ubounds`j''" != "."{
            cap drop if `x' > `ubounds`j''
         }
         replace `f' = `set'*`f'
*---------------------------------
* produce the plot
*---------------------------------
         if `np' > 1 {
            tempfile t`j'
            local s `"saving("`t`j''.gph",replace)"'
            local cs `"`cs' "`t`j''.gph" "'
         }
         twoway (line `f' `x' ) (`addplot'), legend(off) ///
         ytitle("Density") xtitle("`vlab'") `s' `nodraw' `goptions'
*---------------------------------
* find mode
*---------------------------------
         su `f'
         tempvar m`v'
         gen `m`v'' = `x'*(`f' == r(max))
         su `m`v''
         local mode`v' = r(sum)
*---------------------------------
* save plotting points
*---------------------------------
         if "`save'" != "" {
            gen `id' = `j'
            keep `id' `x' `f'
            drop if `f' == .
            if `j' > 1 {
               append using `save'
            }
            save `save' , `dreplace'
         }
         restore
      }
*---------------------------------
* rename variables in saved file
*---------------------------------
      if `"`save'"' != "" {
         preserve
         use `"`save'"' , clear
         rename `id' plot
         rename `f' f
         rename `x' x
         save `"`save'"' , `dreplace'
         restore
      }
*-------------------------
* combined plot
*-------------------------
      if `np' > 1 {
         graph combine `cs' , `cgoptions'
      }
  }
*---------------------------------
* Display and return posterior mode
*---------------------------------
   di as text _dup(35) "-"
   di as txt "Parameter" _col(20) "Posterior mode"
   di as text _dup(35) "-"
   local i = 0
   foreach v of varlist `varlist' {
      di as txt "`v'" _col(20) as res %10.3f `mode`v''
      local ++i
      return scalar mode`i' = `mode`v''
   }
   di as text _dup(35) "-"
   
end
