*============================================================================
* mcmcintervals:  Plot showing MCMC results across successive intervals with the run
* Author:        John Thompson
* Date:          Dec 2012
*============================================================================

program mcmcintervals, sortpreserve
   version 12.1
   syntax varlist [if] [in] , [ ///
   M(integer 10)                /// number of intervals
   Level(integer 80)            /// percent coverage for the range plot
   Chain(varname)               /// variable denoting the chain number when multiple chains are run
   Iteration(varname)           /// variable to control the x-axis
   GOPTions(string asis)        /// options passed to graph twoway
   CGOPTions(string asis)       /// options passed to graph combine
   Save(string)                 /// .dta file to save the plotting points
   ]
   
*---------------------------------
* Check options
*---------------------------------
   if `m' < 1 {
      di as err "mcmcintervals: option m must be positive"
      exit(499)
   }
   if `level' < 1 | `level' > 100 {
      di as err "mcmcintervals: option level must be between 1 and 100"
      exit(499)
   }
*---------------------------------
* Create plot
*---------------------------------
   quietly {
      marksample touse
      preserve
*---------------------------------
* count number of parameters
*---------------------------------
      local np : word count `varlist'
      if `np' > 1 local nodraw "nodraw"
*---------------------------------
* count number of chains
*---------------------------------
      if "`chain'" == "" {
         local nc = 1
      }
      else {
         qui levelsof `chain' if `touse', local(clist)
         local nc : word count `clist'
      }
*---------------------------------
* create x-axis variable
*---------------------------------
      tempvar t h
      if "`iteration'" == "" {
         if "`chain'" == "" gen `t' = _n
         else {
            gen `h' = _n
            sort `chain' `h'
            by `chain': egen `t' = seq()
         }
         local orderlab = "Iteration"
      }
      else {
         gen `t' = `iteration'
         local orderlab : variable label `iteration'
         if "`orderlab'" == "" local orderlab "`iteration'"
      }
      keep if `touse'
      sort `chain' `t'
*---------------------------------
* interval size for each plot
*---------------------------------
      if "`chain'" != "" {
         local n = 0
         local nm = 1E10
         foreach c of local clist {
            su `t' if `chain' == `c' 
            local n = max(`n',r(max))
            local nm = min(`nm',r(min))
         }
      }
      else {
         su `t' 
         local n = r(max)
         local nm = r(min)
      }
      local nm1 = `nm' - 1
      local step = (`n'-`nm1')/`m'
      local scale = int((`n'-`nm1')/5)
*---------------------------------
* details of save file
*---------------------------------
      if "`save'" != "" {
         tokenize `"`save'"',parse(",")
         local save = trim(subinstr(`"`1'"',`"""',"",.))
         local dreplace `"`3'"'
         if `np' > 1 {
            local dreplace `"replace"'
         }
      }
*---------------------------------
* prepare plots for each variable
*---------------------------------
      tempvar y mn lb ub xp par cp mi
      gen `mn' = .
      gen `lb' = .
      gen `ub' = .
      gen `xp' = .
      gen `cp' = .
      gen `mi' = .
      gen str20 `par' = ""
      local lc = (100-`level')/2
      local uc = 100 - `lc'
      local j = 0
      foreach v of varlist `varlist' {
         local ++j
         su `v' , detail
         local ym = r(p50)
         local line = 0
         local low = `nm'
         forvalues k = 1/`m' {
            local high = `low' + `step'
            if `nc' == 1 {
               local ++line
               centile `v' if `t' >= `low' & `t' < `high' , cent(`lc' 50 `uc')
               replace `mn' = r(c_2) in `line'
               replace `lb' = r(c_1) in `line'
               replace `ub' = r(c_3) in `line'
               replace `cp' = 1 in `line'
               replace `par' = "`v'" in `line'
               su `t' if `t' >= `low' & `t' < `high'
               replace `xp' = r(mean) in `line'
               replace `mi' = r(mean) in `line'
            }
            else {
               local ci = 0
               foreach c of local clist {
                  local ++ci
                  local ++line
                  centile `v' if `t' >= `low' & `t' < `high' & `chain' == `c' , cent(`lc' 50 `uc')
                  replace `mn' = r(c_2) in `line'
                  replace `lb' = r(c_1) in `line'
                  replace `ub' = r(c_3) in `line'
                  replace `cp' = `c' in `line'
                  replace `par' = "`v'" in `line'
                  su `t' if `t' >= `low' & `t' < `high' & `chain' == `c' 
                  replace `xp' = r(mean)-`step'*(`ci'-`nc'/2)/10 in `line'
                  replace `mi' = r(mean) in `line'
               }
            }
            local low = `high'
         }
*---------------------------------
* produce the plot
*---------------------------------
         if `np' > 1 {
            tempfile t`j'
            local s = "saving(`t`j''.gph,replace)"
            local cs = "`cs' `t`j''.gph"
         }
         twoway (scatter `mn' `xp' ,mcol(black) )  ///
         (rcap `lb' `ub' `xp' ,lcol(black)) , legend(off) ///
         yline(`ym', lpat(dash) lcol(red) ) ytitle("`level'% Interval for `v'") xtitle("`orderlab'") ///
         `s' xscale(range(`nm1' `n')) xlabel(`nm1'(`scale')`n') `nodraw' `goptions'
         if "`save'" != "" {
            if `j' > 1 {
               append using `save'
            }
            save `save' , `dreplace'
         }
      }
*-------------------------
* combined plot
*-------------------------
      if `np' > 1 {
         graph combine `cs' , `cgoptions'
      }
*-------------------------------
* rename variables in saved data
*-------------------------------
      if "`save'" != "" {
         use `save' , clear
         keep `cp' `par' `mi' `mn' `lb' `ub'
         order `cp' `par' `mi' `mn' `lb' `ub'
         rename `mi' iteration
         rename `mn' median
         rename `lb' lbound
         rename `ub' ubound
         rename `par' parameter
         rename `cp' chain
         keep if chain != .
         save `save' , replace
      }
      restore
   }
end
