*============================================================================
* mcmclength:  Estimate the required run length of an MCMC run
*             using a blocked bootstrap
* Author:     John Thompson
* Date:       Dec 2012
*============================================================================

program mcmclength, rclass
   version 12.1
   syntax varlist [if] [in] , [ ///
   Blocksize(integer -1)        ///  size of blocks for bootstrap
   Target(real -1)              ///  target standard error
   PTarget(real 5)              ///  target se as a percent of the estimate
   Level(cilevel)               ///  size of credible intervals
   BOOTopts(string)             ///  options passed to bootstrap command
   AC(real 0.05)                ///  autocorrelation used to pick blocksize if not given
   ]
   
*---------------------------------
* Check options
*---------------------------------
   if `ptarget' <= 0 {
      di as err "ptarget must be positive"
      exit(499)
   }
   if `ac' <= 0 | `ac' >= 1{
      di as err "ac must be between 0 and 1"
      exit(499)
   }
*---------------------------------
* Perform calculations
*---------------------------------
   preserve
   marksample touse
   qui keep if `touse'
   tempfile dfile
   
   tempvar t x1 block tac
   gen `t' = _n
   qui tsset `t'
   gen `x1' = 0
   qui count
   local n = r(N)
   tempname b var
*--------------------------------------
* Each variable in turn
*--------------------------------------
   quietly save `dfile'
   
   local k = 0
   foreach v of varlist `varlist' {
      use `dfile' , clear
      local ++k
*--------------------------------------
* Estimate blocksize if not given
*--------------------------------------
      if `blocksize' <= 0  {
         qui ac `v' , nograph gen(`tac') lags(100)
         local bs = 1
         local j = 2
         while `tac'[`j'] != . {
            if abs(`tac'[`j']) > `ac' local bs = `j'
            local ++j
         }
         qui drop `tac'
      }
      else {
         local bs = `blocksize'
         if "`ac'" != "" di as err "warning: ac ignored as blocksize specified"
      }
      qui egen `block' = seq() , b(`bs')
*--------------------------------------
* display header
*--------------------------------------
      di " "
      if `target' <= 0 di as res "`v': target standard error is `ptarget'% of the estimate"
      else di as res "`v': target standard error is `target'"
      di as res "Current chain length: `n'"
      di as res "Block size for bootstrap: `bs'"
      di as res _dup(67) "-"
      di as res " range                 estimate    st error      target     updates"
      di as res _dup(67) "-"
*--------------------------------------
* run the bootstrap
*--------------------------------------
      qui bootstrap r(mn1) r(sd1) r(lb1) r(ub1) , cluster(`block') `bootopts' : mcmcstats `v' , level(`level')
*--------------------------------------
* display & return results
*--------------------------------------
      matrix `b' = e(b)
      matrix `var' = e(V)
      if `target' <= 0 local tb = `ptarget'*abs(`b'[1,1])/100
      else local tb = `target'
      local update = `n'*`var'[1,1]/`tb'^2
      di as res %-19s "mean" as res %12.4f `b'[1,1] %12.4f sqrt(`var'[1,1]) ///
      %12.4f `tb' %12.0f `update'
      return scalar nmean`k' = round(`update' )
      if `target' <= 0 local tb = `ptarget'*abs(`b'[1,2])/100
      else local tb = `target'
      local update = `n'*`var'[2,2]/`tb'^2
      di as res %-19s "Standard deviation" as res %12.4f `b'[1,2] %12.4f sqrt(`var'[2,2])  ///
      %12.4f `tb' %12.0f `update'
      return scalar nsd`k' = round(`update')
      if `target' <= 0 local tb = `ptarget'*abs(`b'[1,3])/100
      else local tb = `target'
      local update = `n'*`var'[3,3]/`tb'^2
      di as res %-19s "`level'% CrI lower" as res %12.4f `b'[1,3] %12.4f sqrt(`var'[3,3])  ///
      %12.4f `tb' %12.0f `update'
      return scalar nlb`k' = round(`update')
      if `target' <= 0 local tb = `ptarget'*abs(`b'[1,4])/100
      else local tb = `target'
      local update = `n'*`var'[4,4]/`tb'^2
      di as res %-19s "`level'% CrI upper" as res %12.4f `b'[1,4] %12.4f sqrt(`var'[4,4])  ///
      %12.4f `tb' %12.0f `update'
      return scalar nub`k' = round(`update')
      di as res _dup(67) "-"
      di as res "note: if the current chain is thinned then multiply the projected"
      di as res " number of updates by that thinning value"
      drop `block'
   }
   di as txt " "
   restore
end
