*============================================================================
* mcmctrace:  Trace (history) plots of an MCMC analysis
* Author:    John Thompson
* Date:      Dec 2012
*============================================================================

program mcmctrace, sortpreserve
   version 12.1
   syntax varlist [if] [in] , [ ///
   GOPTions(string asis)        /// options passed to graph twoway
   CGOPTions(string asis)       /// options passed to graph combine
   Iteration(varname)           /// variable for the x-axis
   Chain(varname)               /// variable denoting the chain number
   Level(cilevel)               /// level for credible limits
   OVERlay                      /// overlay the traces of all chains onto one graph
   ]
   marksample touse

*---------------------------------
* check options
*---------------------------------
   if "`chain'" == "" & "`overlay'" != "" {
      di as err "warning: overlay ignored - only works with multiple chains"
      local overlay ""
   }
*---------------------------------
* count number of parameters
*---------------------------------
   local np : word count `varlist'
*---------------------------------
* count number of chains
*---------------------------------
   if "`chain'" == "" {
      local nc = 1
   }
   else {
      qui levelsof `chain' if `touse', local(clist)
      local nc : word count `clist'
   }
   if "`overlay'" == "" {
      local np = `np'*`nc'
   }
   if `np' > 1 local nodraw "nodraw"
*---------------------------------
* create x-axis variable
*---------------------------------
   tempvar t h
   if "`iteration'" == "" {
      if "`chain'" == "" gen `t' = _n
      else {
         gen `h' = _n
         sort `chain' `h'
         by `chain': egen `t' = seq()
      }
      local orderlab = "Iteration"
   }
   else {
      gen `t' = `iteration'
      local orderlab : variable label `iteration'
      if "`orderlab'" == "" local orderlab "`iteration'"
   }
*---------------------------------
* Graph for each variable
*---------------------------------
   sort `chain' `t'
   local i = 0
   local cs = ""
   foreach v of varlist `varlist' {
*---------------------------------
* plot each chain
*---------------------------------
      if "`chain'" == "" {
*---------------------------------
* Single chain
*---------------------------------
         local i = `i' + 1
         tempfile t`i'
         local s = "saving(`t`i''.gph,replace)"
         local cs "`cs' `t`i''.gph"
*---------------------------------
* find median & credible interval
*---------------------------------
         qui centile `v' if `touse'
         local md = r(c_1)
         local cl = (100-`level')/2
         qui centile `v' if `touse' , cen(`cl')
         local lb = r(c_1)
         local cl = 100-`cl'
         qui centile `v' if `touse' , cen(`cl')
         local ub = r(c_1)
         qui line `v' `t' if `touse' , xtitle("`orderlab'") yline(`md' , lp(solid)) ///
         yline(`lb' , lp(dash)) yline(`ub' , lp(dash)) `goptions' `s' `nodraw'
      }
      else {
         if "`overlay'" != "" {
*---------------------------------
* Several chains - overlayed
*---------------------------------
            local i = `i' + 1
            tempfile t`i'
            local s = "saving(`t`i''.gph,replace)"
            local cs "`cs' `t`i''.gph"
*---------------------------------
* find credible intervals
*---------------------------------
            qui centile `v' if `touse'
            local md = r(c_1)
            qui centile `v' if `touse' , cen(2.5)
            local lb = r(c_1)
            qui centile `v' if `touse' , cen(97.5)
            local ub = r(c_1)
            local pc ""
            forvalues c = 1 / `nc' {
               local pc "`pc' (line `v' `t' if `touse' & chain == `c')"
            }
            qui twoway `pc' , xtitle("`orderlab'") yline(`md' , lp(solid)) ///
            yline(`lb' , lp(dash)) yline(`ub' , lp(dash)) legend(off) `s' `nodraw' `goptions'
         }
         else {
*---------------------------------
* Several chains - not overlayed
*---------------------------------
            foreach c of local clist {
*---------------------------------
* find credible intervals
*---------------------------------
               qui centile `v' if `touse' & chain == `c'
               local md = r(c_1)
               qui centile `v' if `touse' & chain == `c' , cen(2.5)
               local lb = r(c_1)
               qui centile `v' if `touse' & chain == `c' , cen(97.5)
               local ub = r(c_1)
               local i = `i' + 1
               tempfile t`i'
               local s = "saving(`t`i''.gph,replace)"
               local cs "`cs' `t`i''.gph"
               qui line `v' `t' if `touse' & chain == `c' , xtitle("`orderlab' (chain `c')") yline(`md' , lp(solid)) ///
               yline(`lb' , lp(dash)) yline(`ub' , lp(dash)) `s' `nodraw' `goptions'
            }
         }
      }
   }
*---------------------------------
* combine plots if needed
*---------------------------------
   if `np' > 1 {
      qui graph combine `cs' , `cgoptions'
   }
end
