*------------------------------
* Code for Chapter 2
* data file: swiss.dta
*------------------------------
clear
set scheme sj
set more off
program drop _all
pause off
*-------------------------------
* Section 2.2
* Prior, Likelihood & Posterior
* figure 2.1 & code to dof
*-------------------------------
range theta 0 1 1000
gen Prior = 5*inrange(theta,0.3,0.5)
gen LogL = 5*log(theta)+5*log(1-theta)
summarize LogL
gen Likelihood = exp(LogL-r(max))
integ Likelihood theta
replace Likelihood = Likelihood/r(integral)
gen Posterior = Likelihood*Prior
integ Posterior theta
replace Posterior = Posterior/r(integral)
twoway (line Prior theta) (line Likelihood theta) ///
   (line Posterior theta), legend(pos(1) ring(0) row(3))
* FIGURE 2.1  
graph export "eps/chap2_1.eps", replace
pause "Press q to continue"
*-------------------------------
* Section 2.4
* Blood pressure & age in Hungary
* figure 2.2
*-------------------------------
use data/hungarian_all.dta, clear
rename v1_3 Age
rename v2_3 SBP
replace SBP = . if SBP <= 0
twoway (lfitci SBP Age , lpat(solid)) ///
  (scatter SBP Age , msymb(circle)) , ///
  ytitle(SBP) xscale(range(27 67)) xlabel(30(10)60) legend(off)
* FIGURE 2.2  
graph export "eps/hungary.eps", replace
pause "Press q to continue"
*-------------------------------
* Section 2.4
* Blood pressure & age
* figure 2.3
*-------------------------------
program logpost
    args logp b

    local alpha = `b'[1,1]
    local beta  = `b'[1,2]
    local sigma = `b'[1,3]
    scalar `logp' = 0
    logdensity normal  `logp'  sbp   `alpha'+`beta'*age `sigma'
    logdensity normal  `logp' `alpha' 105 10
    logdensity normal  `logp' `beta'  0.55 0.3
    logdensity uniform `logp' `sigma' 5 35
end    

use data/swiss.dta, clear
drop if missing(sbp,age)
matrix b = (100,.,20)
tempname pf
postfile `pf' beta logp using temp/temp.dta, replace
foreach beta of numlist 0.4(0.001)0.8 {
   matrix b[1,2] = `beta'
   logpost logf b 
   post `pf' (`beta') (logf)
}
postclose `pf'

use temp/temp.dta, clear	
line logp beta , name(logp,replace) ytitle(Log-posterior)
summarize logp
gen p = exp(logp - r(max))
integ p beta
replace p = p/r(integral)
line p beta , name(posterior,replace) ytitle(Posterior)

*------------------------------------------
* Create figure 2.3 (code not in book)
*------------------------------------------
graph display logp
graph save temp/plot1.gph, replace
graph display posterior
graph save temp/plot2.gph, replace
graph drop _all
graph combine temp/plot1.gph temp/plot2.gph
* FIGURE 2.2  
graph export "eps/chap2_logp.eps", replace
pause "Press q to continue"

*------------------------------------------
* USING faster logpost
*------------------------------------------
program drop _all
program logpost
    args logp b

    local beta  = `b'[1,2]
    tempvar z
    gen `z' = -(sbp-100-`beta'*age)^2/800
    qui summarize `z' , meanonly
    scalar `logp' = r(sum) - (`beta'-0.55)^2/0.18
end    

use data/swiss.dta, clear
drop if missing(sbp,age)
matrix b = (100,.,20)
tempname pf
postfile `pf' beta logp using temp/temp.dta, replace
foreach beta of numlist 0.4(0.001)0.8 {
   matrix b[1,2] = `beta'
   logpost logf b 
   post `pf' (`beta') (logf)
}
postclose `pf'

use temp/temp.dta, clear	
line logp beta , name(logp,replace) ytitle(Log-posterior)
summarize logp
gen p = exp(logp - r(max))
integ p beta
replace p = p/r(integral)
line p beta , name(posterior,replace) ytitle(Posterior)

graph display logp
graph save temp/plot1.gph, replace
graph display posterior
graph save temp/plot2.gph, replace
graph drop _all
graph combine temp/plot1.gph temp/plot2.gph
pause "Press q to continue"

*-------------------------------
* Section 2.5
* Compare hierarchical & non-hierarchical
* figure 2.4
*-------------------------------
program drop _all
*---------------------------------
* non-Hierarchical
*---------------------------------
program logpost1
    args logp b

    local alpha = `b'[1,1]
    local beta  = `b'[1,2]
    local sigma = `b'[1,3]
    scalar `logp' = 0
    logdensity normal  `logp'  sbp   `alpha'+`beta'*age `sigma' if pain == 1
    logdensity normal  `logp' `alpha' 105 10
    logdensity normal  `logp' `beta'  0.55 0.3
    logdensity uniform `logp' `sigma' 5 35
end    
*---------------------------------
* Hierarchical
*---------------------------------
program logpost
    args logp b

    local alpha1 = `b'[1,1]
    local beta1  = `b'[1,2]
    local sigma1 = `b'[1,3]
    local mub    = `b'[1,4]
    local sigmab = `b'[1,5]
    scalar `logp' = 0
    logdensity normal `logp' sbp `alpha1'+`beta1'*age `sigma1' if pain==1
    logdensity normal `logp' `beta1' `mub' `sigmab'
end

use data/swiss.dta, clear
drop if missing(sbp,age)
matrix b = (100,.,20,0.85,0.1)
tempname pf
postfile `pf' beta logp logph using temp/temp.dta, replace
foreach beta of numlist 0(0.001)1.2 {
   matrix b[1,2] = `beta'
   logpost1 logf b 
   logpost logfh b
   post `pf' (`beta') (logf) (logfh)
}
postclose `pf'

use temp/temp.dta, clear	
summarize logp
gen p = exp(logp - r(max))
integ p beta
replace p = p/r(integral)
summarize logph
gen h = exp(logph - r(max))
integ h beta
replace h = h/r(integral)
twoway (line p beta) (line h beta) , leg(off) ///
   xscale(range(0 1.2)) xlabel(0(0.2)1.2)
* FIGURE 2.4  
graph export "eps/chap2_logp1.eps", replace
pause "Press q to continue"

*-------------------------------
* Section 2.6
* General posteriors
*-------------------------------
*--------------------------
* ipar - output to dof and test
*--------------------------
program drop _all
program logpost
    args logp b ipar

    local alpha = `b'[1,1]
    local beta  = `b'[1,2]
    local sigma = `b'[1,3]
    scalar `logp' = 0
    logdensity normal `logp' sbp "`alpha' + `beta'*age" `sigma'
    if `ipar' == 1      logdensity normal  `logp' `alpha' 105 10
    else if `ipar' == 2 logdensity normal  `logp' `beta'  0.55 0.3
    else if `ipar' == 3 logdensity uniform `logp' `sigma' 5 35
end  

use data/swiss.dta, clear
drop if missing(sbp,age)
matrix b = (100,.,20)
tempname pf
postfile `pf' beta logp using temp/temp.dta, replace
foreach beta of numlist 0.4(0.001)0.8 {
   matrix b[1,2] = `beta'
   logpost logf b 2
   post `pf' (`beta') (logf)
}
postclose `pf'

use temp/temp.dta, clear	
line logp beta , name(logp,replace) ytitle(Log-posterior)
summarize logp
gen p = exp(logp - r(max))
integ p beta
replace p = p/r(integral)
line p beta , name(posterior,replace) ytitle(Posterior)

graph display logp
graph save temp/plot1.gph, replace
graph display posterior
graph save temp/plot2.gph, replace
graph drop _all
graph combine temp/plot1.gph temp/plot2.gph
pause "Press q to continue"

*--------------------------
* ifin
*--------------------------
program drop _all
program logpost
    syntax anything [fweight] [if] [in]

    marksample touse
    tokenize `"`anything'"'
    local logp "`1'"
    local b "`2'"
    local ipar = `3'

    local alpha = `b'[1,1]
    local beta  = `b'[1,2]
    local sigma = `b'[1,3]
    scalar `logp' = 0
    logdensity normal  `logp' sbp `alpha'+`beta'*age `sigma' [`weight'`exp'] if `touse'
    if `ipar' == 1      logdensity normal  `logp' `alpha' 105 10
    else if `ipar' == 2 logdensity normal  `logp' `beta'  0.55 0.3
    else if `ipar' == 3 logdensity uniform `logp' `sigma' 5 35
end    
use data/swiss.dta, clear
drop if missing(sbp,age)
matrix b = (100,.,20)
tempname pf
postfile `pf' beta logp using temp/temp.dta, replace
foreach beta of numlist 0.4(0.001)0.8 {
   matrix b[1,2] = `beta'
   logpost logf b 2
   post `pf' (`beta') (logf)
}
postclose `pf'

use temp/temp.dta, clear	
line logp beta , name(logp,replace) ytitle(Log-posterior)
summarize logp
gen p = exp(logp - r(max))
integ p beta
replace p = p/r(integral)
line p beta , name(posterior,replace) ytitle(Posterior)

graph display logp
graph save temp/plot1.gph, replace
graph display posterior
graph save temp/plot2.gph, replace
graph drop _all
graph combine temp/plot1.gph temp/plot2.gph
pause "Press q to continue"

*-------------------------------
* Section 2.7
* log-density
*-------------------------------
program drop _all
program logdensity
    syntax anything [fweight] [if] [in]
    version 12
    local distribution "`1'"
    local logp "`2'"
    capture confirm matrix `3'
    if !_rc {
       qui logdensity_`distribution' "M" `0'
    }
    else {
       capture confirm variable `3'
       if !_rc {
           marksample touse
           markout `touse' `3'
           tempname lnf
           qui logdensity_`distribution' "V"  `touse' `lnf' `0'
           qui summarize `lnf' [`weight'`exp'] if `touse' , meanonly
           scalar `logp' = scalar(`logp') + r(sum)
           local n = r(N)
           qui summarize `touse' [`weight'`exp'] if `touse' , meanonly
           if r(N) != `n' scalar `logp' = .
       }
       else {
           tempname slogp
           scalar `slogp' = scalar(`logp')
           qui logdensity_`distribution' "S" `0'
           scalar `logp' = `slogp' + scalar(`logp')
       }
    }
end
  
program logdensity_normal
    local type "`1'"

    if "`type'" == "V" {
        tempvar u 
        local touse `2'
        local lnf `3'
        local y (`6')
        local mu (`7')
        local sd (`8')
        gen `u' = (`y'-`mu')/`sd' if `touse'
        gen `lnf' = -log(`sd') - 0.5*`u'*`u' if `touse'
    }
    else if "`type'" == "S" {
        tempname y mu sd u 
        local logp `3'
        scalar `y' = `4'
        scalar `mu' = `5'
        scalar `sd' = `6'
        scalar `u' = (`y'-`mu') /`sd'
        scalar `logp' = - log(`sd') - 0.5*`u'*`u'
    }
    else {
        display as err "No normal logdensity for data in a matrix"
        exit 198
    }
end

program logpost
    syntax anything [fweight] [if] [in]

    marksample touse
    tokenize `"`anything'"'
    local logp "`1'"
    local b "`2'"
    local ipar = `3'

    local alpha = `b'[1,1]
    local beta  = `b'[1,2]
    local sigma = `b'[1,3]
    scalar `logp' = 0
    logdensity normal  `logp' sbp `alpha'+`beta'*age `sigma' [`weight'`exp'] if `touse'
    if `ipar' == 1      logdensity normal  `logp' `alpha' 105 10
    else if `ipar' == 2 logdensity normal  `logp' `beta'  0.55 0.3
    else if `ipar' == 3 logdensity uniform `logp' `sigma' 5 35
end    
use data/swiss.dta, clear
drop if missing(sbp,age)
matrix b = (100,.,20)
tempname pf
postfile `pf' beta logp using temp/temp.dta, replace
foreach beta of numlist 0.4(0.001)0.8 {
   matrix b[1,2] = `beta'
   logpost logf b 2
   post `pf' (`beta') (logf)
}
postclose `pf'

use temp/temp.dta, clear	
line logp beta , name(logp,replace) ytitle(Log-posterior)
summarize logp
gen p = exp(logp - r(max))
integ p beta
replace p = p/r(integral)
line p beta , name(posterior,replace) ytitle(Posterior)

graph display logp
graph save temp/plot1.gph, replace
graph display posterior
graph save temp/plot2.gph, replace
graph drop _all
graph combine temp/plot1.gph temp/plot2.gph
pause "Press q to continue"


