set more off
set matastrict off
mata:
mata clear
mata mlib create libmcmc , dir(PERSONAL) replace
/************************************************************************************
MHSNORM
metropolis algorithm based on normal random walk
if logp is not missing then it is assumed known
*/
void mhsnorm(
real scalar      logp,
pointer(function) userprog,
real matrix      X,
real rowvector   theta,
real scalar      p,
real scalar      sd,
|  real scalar      accept)
{
   real scalar oldtheta, newlogp
   oldtheta = theta[p]
   if ( logp == . ) {
      logp = (*userprog)(X,theta,p)
      if ( logp == . ) {
         printf("mhsnorm: Unable to calculate log-posterior for parameter %4.0f\n",p)
         matrixprint(" %10.4f",theta)
         exit(1400)
      }
   }
   theta[p] = rnormal(1,1,theta[p],sd)
   newlogp = (*userprog)(X,theta,p)
   if ( newlogp == . ) {
      printf("mhsnorm: Unable to calculate log-posterior for parameter %4.0f\n",p)
      matrixprint(" %10.4f",theta)
      exit(1400)
   }
   if (  log(runiform(1,1)) > (newlogp-logp) ) {
      theta[p] = oldtheta
      if ( args() == 7 ) accept = 0
   }
   else {
      if ( args() == 7 ) accept = 1
      logp = newlogp
   }
}
mata mlib add libmcmc mhsnorm()
/************************************************************************************
MHMSNORM
metropolis algorithm based on multivariate normal random walk
if logp is not missing then it is assumed known
*/
void mhsmnorm(
real scalar      logp,
pointer(function) userprog,
real matrix      X,
real rowvector   theta,
real scalar      p,
real matrix      C,
|  real scalar      accept)
{
   real scalar  newlogp, d, pe
   real rowvector oldtheta
   
   d = rows(C)
   pe = p + d  - 1
   
   oldtheta = theta
   if ( logp == . ) {
      logp = (*userprog)(X,theta,p)
      if ( logp == . ) {
         printf("mhsmnorm: Unable to calculate log-posterior for parameters %4.0f-%4.0f\n",p,pe)
         matrixprint(" %10.4f",theta)
         exit(1400)
      }
   }
   theta[|p\pe|] = theta[|p\pe|] + (C*rnormal(d,1,0,1))'
   newlogp = (*userprog)(X,theta,p)
   if ( newlogp == . ) {
      printf("mhsmnorm: Unable to calculate log-posterior for parameters %4.0f-%4.0f\n",p,pe)
      matrixprint(" %10.4f",theta)
      exit(1400)
   }
   if ( log(runiform(1,1)) > (newlogp-logp) ) {
      theta = oldtheta
      if ( args() == 7 ) accept = 0
   }
   else {
      if ( args() == 7 ) accept = 1
      logp = newlogp
   }
}
mata mlib add libmcmc mhsmnorm()
/************************************************************************************
MHSLOGN
metropolis algorithm based on log-normal random walk
*/
void mhslogn(
real scalar      logp,
pointer(function) userprog,
real matrix      X,
real rowvector   theta,
real scalar      p,
real scalar      sd,
|  real scalar      accept)
{
   real scalar oldtheta, newlogp
   
   oldtheta = theta[p]
   if ( logp == . ) {
      logp = (*userprog)(X,theta,p)
      if ( logp == . ) {
         printf("mhslogn: Unable to calculate log-posterior for parameter %4.0f\n",p)
         matrixprint(" %10.4f",theta)
         exit(1400)
      }
   }
   delta = exp( rnormal(1,1,0,sd) )
   theta[p] = theta[p]*delta
   newlogp = (*userprog)(X,theta,p)
   if ( newlogp == . ) {
      printf("mhslogn: Unable to calculate log-posterior for parameter %4.0f\n",p)
      matrixprint(" %10.4f",theta)
      exit(1400)
   }
   if (  log(runiform(1,1)) > (newlogp+log(theta[p]))-(logp+log(oldtheta)) ) {
      theta[p] = oldtheta
      if ( args() == 7 ) accept = 0
   }
   else {
      if ( args() == 7 ) accept = 1
      logp = newlogp
   }
}
mata mlib add libmcmc mhslogn()
/************************************************************************************
MHSTRNC
metropolis algorithm based on transformed normal random walk
*/
void mhstrnc(
real scalar      logp,
pointer(function) userprog,
real matrix      X,
real rowvector   theta,
real scalar      p,
real scalar      sd,
real scalar      low,
real scalar      high,
| real scalar      accept)
{
   oldtheta = theta[p]
   if ( logp == . ) {
      logp = (*userprog)(X,theta,p)
      if ( logp == . ) {
         printf("mhstrnc: Unable to calculate log-posterior for parameter %4.0f\n",p)
         matrixprint(" %10.4f",theta)
         exit(1400)
      }
   }
   if ( low == . ) {
      if ( high == . ) {
         delta = rnormal(1,1,0,sd)
         theta[p] = oldtheta+delta
         scale1 = 0
         scale2 = 0
      }
      else {
         delta = exp(rnormal(1,1,0,sd))
         theta[p] = high - (high-oldtheta)/delta
         scale1 = log(high-oldtheta)
         scale2 = log(high-theta[p])
      }
   }
   else {
      if ( high == . ) {
         delta = exp(rnormal(1,1,0,sd))
         theta[1,p] = low + (oldtheta-low)*delta
         scale1 = log(oldtheta-low)
         scale2 = log(theta[p]-low)
      }
      else {
         delta = exp(rnormal(1,1,0,sd))
         theta[1,p] = (high*(oldtheta-low)*delta+low*(high-oldtheta))/(high-oldtheta+(oldtheta-low)*delta)
         scale1 = log(high-oldtheta)+log(oldtheta-low)
         scale2 = log(high-theta[p])+log(theta[p]-low)
      }
   }
   newlogp = (*userprog)(X,theta,p)
   if ( newlogp == . ) {
      printf("mhstrnc: Unable to calculate log-posterior for parameter %4.0f\n",p)
      matrixprint(" %10.4f",theta)
      exit(1400)
   }
   if (  log(runiform(1,1)) > (newlogp+scale2)-(logp+scale1) ) {
      theta[p] = oldtheta
      if ( args() == 9 ) accept = 0
   }
   else {
      if ( args() == 9 ) accept = 1
      logp = newlogp
   }
}
mata mlib add libmcmc mhstrnc()
/************************************************************************************
GBSSLICE
general slice sampler
*/
void gbsslice(
real scalar      logp,
pointer(function) userprog,
real matrix      X,
real rowvector   theta,
real scalar      p,
real scalar      stepsize,
real scalar      nstep,
real scalar      lbound,
real scalar      ubound,
real scalar      maxtry,
|   real scalar      fail,
real scalar      t1,
real scalar      t2 )
{
   gbsslice_generic("m","","",logp,"",theta,p,stepsize,nstep,lbound,ubound,maxtry,fail=0,t1=0,t2=0,userprog,X)
}
mata mlib add libmcmc gbsslice()
/************************************************************************************
GBSSLICE
general slice sampler for STATA
*/
void gbsslice_stata(
string scalar cmd,
string scalar    namelogp,
real scalar      logp,
string scalar    b,
real scalar      p,
real scalar      stepsize,
real scalar      nstep,
real scalar      lbound,
real scalar      ubound,
real scalar      maxtry )
{
   t = st_matrix(b)
   gbsslice_generic("s",cmd,namelogp,logp,b,t,p,stepsize,nstep,lbound,ubound,maxtry,fail=0,t1=0,t2=0)
   st_matrix(b,t)
   st_local("try1",strofreal(t1))
   st_local("try2",strofreal(t2))
   st_local("fail",strofreal(fail))
   st_local("logp",strofreal(logp))
}
mata mlib add libmcmc gbsslice_stata()
/************************************************************************************
GBSSLICE
general slice sampler for STATA or MATA
*/
void gbsslice_generic(
string scalar source,
string scalar cmd,
string scalar namelogp,
real scalar      logp,
string scalar    nametheta,
real rowvector   theta,
real scalar      p,
real scalar      stepsize,
real scalar      nstep,
real scalar      lbound,
real scalar      ubound,
real scalar      maxtry,
|   real scalar      fail,
real scalar      t1,
real scalar      t2,
pointer(function) userprog,
real matrix      X)
{
   oldx = theta[p]
   if ( lbound == . ) lbound = oldx  - 2*nstep*stepsize
   if ( ubound == . ) ubound = oldx  + 2*nstep*stepsize
   if ( logp == . ) {
      if ( source == "m" ) logp = (*userprog)(X,theta,p)
      else {
         st_matrix(nametheta,theta)
         stata(cmd,1)
         logp = st_numscalar(namelogp)
      }
      if ( logp == . ) {
         printf("gbsslice: Unable to calculate log-posterior for parameter %4.0f\n",p)
         matrixprint(" %10.4f",theta)
         exit(1400)
      }
   }
   y = logp + log(runiform(1,1))
   ub = oldx + stepsize*runiform(1,1)
   if ( ub > ubound ) ub = (ubound+oldx)/2
   theta[p] = ub
   if ( source == "m" ) yub = (*userprog)(X,theta,p)
   else {
      st_matrix(nametheta,theta)
      stata(cmd,1)
      yub = st_numscalar(namelogp)
   }
   if ( yub == . ) {
      printf("gbsslice: Unable to calculate log-posterior for parameter %4.0f\n",p)
      matrixprint(" %10.4f",theta)
      exit(1400)
   }
   lb = ub - stepsize
   if ( lb < lbound ) lb = (lbound+oldx)/2
   theta[p] = lb
   if ( source == "m" ) ylb = (*userprog)(X,theta,p)
   else {
      st_matrix(nametheta,theta)
      stata(cmd,1)
      ylb = st_numscalar(namelogp)
   }
   if ( ylb == . ) {
      printf("gbsslice: Unable to calculate log-posterior for parameter %4.0f\n",p)
      matrixprint(" %10.4f",theta)
      exit(1400)
   }
   k = floor(nstep*runiform(1,1))
   j = nstep - k
   try1 = 0
   while( k > 0 & yub > y ) {
      k--
      ub = ub + stepsize
      if ( ub > ubound ) ub = (ubound + ub - stepsize)/2
      theta[p] = ub
      if ( source == "m" ) yub = (*userprog)(X,theta,p)
      else {
         st_matrix(nametheta,theta)
         stata(cmd,1)
         yub = st_numscalar(namelogp)
      }
      try1++
      if ( yub == . ) {
         printf("gbsslice: Unable to calculate log-posterior for parameter %4.0f\n",p)
         matrixprint(" %10.4f",theta)
         exit(1400)
      }
   }
   while( j > 0 & ylb > y ) {
      j--
      lb = lb - stepsize
      if ( lb < lbound ) lb = (lbound + lb + stepsize)/2
      theta[p] = lb
      if ( source == "m" ) ylb = (*userprog)(X,theta,p)
      else {
         st_matrix(nametheta,theta)
         stata(cmd,1)
         ylb = st_numscalar(namelogp)
      }
      if ( ylb == . ) {
         printf("gbsslice: Unable to calculate log-posterior for parameter %4.0f\n",p)
         matrixprint(" %10.4f",theta)
         exit(1400)
      }
      try1++
   }
   try2 = 0
   yhit = y - 1
   while( yhit < y  & try2 < maxtry ) {
      theta[p] = lb + (ub-lb)*runiform(1,1)
      if ( source == "m" ) yhit = (*userprog)(X,theta,p)
      else {
         st_matrix(nametheta,theta)
         stata(cmd,1)
         yhit = st_numscalar(namelogp)
      }
      if ( yhit == . ) {
         printf("gbsslice: Unable to calculate log-posterior for parameter %4.0f\n",p)
         matrixprint(" %10.4f",theta)
         exit(1400)
      }
      if ( theta[p] > oldx ) ub = theta[p]
      else lb = theta[p]
      try2++
   }
   if ( yhit < y ) {
      fail = 1
      logp = .
   }
   else {
      fail = 0
      logp = yhit
   }
   if ( args() > 12 ) t1 = try1
   if ( args() > 13 ) t2 = try2
}
mata mlib add libmcmc gbsslice_generic()
/************************************************************************************
GBSGRIDDY
general griddy sampler for MATA
*/
void gbsgriddy(
real scalar logp,
pointer(function) userprog,
real matrix      X,
real rowvector   theta,
real scalar      p,
string scalar    g,
| real scalar      metropolis,
real scalar      histogram)
{
   gbsgriddy_generic("m","","",logp,"",theta,p,g,metropolis,histogram,userprog,X)
}
mata mlib add libmcmc gbsgriddy()
/************************************************************************************
GBSGRIDDY_STATA
Calls griddy sampler from Stata
*/
void gbsgriddy_stata(
string scalar cmd,
string scalar    namelogp,
real scalar      logp,
string scalar    b,
real scalar      p,
string scalar    g,
| real scalar      metropolis,
real scalar      histogram)
{
   t = st_matrix(b)
   gbsgriddy_generic("s",cmd,namelogp,logp,b,t,p,g,metropolis,histogram)
   st_matrix(b,t)
   st_local("logp",strofreal(logp))
}
mata mlib add libmcmc gbsgriddy_stata()
/************************************************************************************
GBSGRIDDY_GENERIC
general griddy sampler for MATA or STATA
*/
void gbsgriddy_generic(
string scalar source,
string scalar cmd,
string scalar    namelogp,
real scalar logp,
string scalar    nametheta,
real rowvector   theta,
real scalar      p,
string scalar    g,
| real scalar      metropolis,
real scalar      histogram,
pointer(function) userprog,
real matrix      X)
{
   real scalar j, maxy, h, pa, u, r, c1, c2, c3, oldlogp, oldx
   real rowvector grid, f, y
   
   if ( args() <  8 ) metropolis = 0
   if ( args() <  9 ) histogram = 0
   /*---------------------------------
   Save current point
   ---------------------------------*/
   oldx = theta[p]
   oldlogp = logp
   /*-----------------------------------
   calculate over a given grid
   or Create a grid from the end points
   -----------------------------------*/
   glist = tokens(g)
   npts = length(glist)
   grid = J(1,npts,0)
   for (j=1;j<=npts;j++) {
      grid[j] = strtoreal(glist[j])
      if ( glist[j] == "." ) setgrid = 1
   }
   f = J(1,npts,0)
   y = J(1,npts,0)
   maxy = 0
      for (j=1;j<=npts;j++) {
         theta[p] = grid[j]
         if ( source == "m" ) y[j] = (*userprog)(X,theta,p)
         else {
            st_matrix(nametheta,theta)
            stata(cmd,1)
            y[j] = st_numscalar(namelogp)
         }
         if ( hasmissing(y[j] ) ) {
            printf("gbsgriddy: Unable to calculate log-posterior for parameter %4.0f\n",p)
            matrixprint(" %10.4f",theta)
            exit(1400)
         }
         if ( j == 1 | y[j] > maxy ) maxy = y[j]
      }
      for (j=2;j<npts;j++) {
         f[j] = exp( y[j]-maxy )
      }
   /*---------------------------------
   Straight line approximation
   ---------------------------------*/
   area = J(1,npts,0)
   if ( histogram == 0 ) {
      totalarea = 0
      for (j=2;j<=npts;j++) {
         h = j - 1
         area[j] = 0.5*(f[j]+f[h])*(grid[j]-grid[h])
         totalarea = totalarea + area[j]
      }
      u = runiform(1,1)*totalarea
      j = 2
      pa = area[2]
      while( u > pa ) {
         j++
         pa = pa + area[j]
      }
      r = (u-pa+area[j])/area[j]
      h = j - 1
      if ( abs(f[j]-f[h]) < 0.000001 ) {
         xr = grid[h]+r*(grid[j]-grid[h])
         yr = f[h]+r*(f[j]-f[h])/(grid[j]-grid[h])
      }
      else {
         c1 = (f[j]-f[h])/(grid[j]-grid[h])
         c2 = 2*f[h]
         c3 = -2*r*area[j]
         d = (-c2+sqrt(c2*c2-4*c1*c3))/(2*c1)
         xr = grid[h] + d
         yr = f[h] + d*(f[j]-f[h])/(grid[j]-grid[h])
      }
   }
   if ( xr == . ) {
      printf("gbsgriddy: Unable to generate random point for parameter %4.0f using the grid\n",p)
      for (j=1;j<=npts;j++) {
         printf("%4.0f: %10.5f %10.5f\n", j,grid[j],f[j])
      }
      exit(1400)
   }
   /*---------------------------------
   Histogram approximation
   ---------------------------------*/
   else {
      xl = J(1,npts,0)
      xh = J(1,npts,0)
      ht = J(1,npts,0)
      xl[1] = grid[1]
      xh[1] = (grid[1]+grid[2])/2
      ht[1] = (f[1]+0.25*(f[2]-f[1]))
      area[1] = ht[1]*(xh[1]-xl[1])
      totalarea = area[1]
      for (j=2;j<npts;j++) {
         xl[j] = (grid[j]+grid[j-1])/2
         xh[j] = (grid[j]+grid[j+1])/2
         ht[j] = f[j]
         area[j] = ht[j]*(xh[j]-xl[j])
         totalarea = totalarea + area[j]
      }
      xh[npts] = grid[npts]
      xl[npts] = (grid[npts]+grid[npts-1])/2
      ht[npts] = f[npts]+0.25*(f[npts-1]-f[npts])
      area[npts] = ht[npts]*(xh[npts]-xl[npts])
      totalarea = totalarea + area[npts]
      
      u = runiform(1,1)*totalarea
      j = 1
      pa = area[1]
      while( u > pa ) {
         j++
         pa = pa + area[j]
      }
      r = (u-pa+area[j])/area[j]
      xr = xl[j] + r*(xh[j]-xl[j])
      yr = ht[j]
   }
   if ( xr == . ) {
      printf("gbsgriddy: Unable to generate random point for parameter %4.0f using the grid\n",p)
      for (j=1;j<=npts;j++) {
         printf("%4.0f: %10.5f %10.5f\n", j,grid[j],f[j])
      }
      exit(1400)
   }
   theta[p] = xr
   if ( source == "m" ) logp = (*userprog)(X,theta,p)
   else {
      st_matrix(nametheta,theta)
      stata(cmd,1)
      logp = st_numscalar(namelogp)
   }
   if ( logp == . ) {
      printf("gbsgriddy: Unable to calculate log-posterior for parameter %4.0f\n",p)
      matrixprint(" %10.4f",theta)
      exit(1400)
   }
   /*---------------------------------
   Metropolis Step
   ---------------------------------*/
   if ( metropolis == 1 ) {
      if ( oldlogp == .  ) {
         theta[p] = oldx
         if ( source == "m" ) oldlogp = (*userprog)(X,theta,p)
         else {
            st_matrix(nametheta,theta)
            stata(cmd,1)
            oldlogp = st_numscalar(namelogp)
         }
         if ( oldlogp == . ) {
            printf("gbsgriddy: Unable to calculate log-posterior for parameter %4.0f\n",p)
            matrixprint(" %10.4f",theta)
            exit(1400)
         }
      }
      if ( histogram == 0 ) {
         j = 2
         while ( oldx > grid[j] ) j++
         oldy = f[j-1] + (oldx - grid[j-1])*(f[j]-f[j-1])/(grid[j]-grid[j-1])
      }
      else {
         j = 1
         while( oldx > xh[j] ) j++
         oldy = ht[j]
      }
      if ( log(runiform(1,1)) < logp-oldlogp-log(yr)+log(oldy) ) theta[p] = xr
      else {
         theta[p] = oldx
         logp = oldlogp
      }
   }
}
mata mlib add libmcmc gbsgriddy_generic()
/************************************************************************************
GBSARMS
general adaptive rejection with metropolis step
*/
void gbsarms(
real scalar      logp,
pointer(function) userprog,
real matrix      X,
real rowvector   theta,
real scalar      p,
string scalar    g,
real scalar      maxtry,
real scalar      minx,
real scalar      maxx,
| real scalar      trys,
real scalar      fail)
{
   gbsarms_generic("m","","",logp,"",theta,p,g,maxtry,minx,maxx,trys,fail,userprog,X)
}
mata mlib add libmcmc gbsarms()
/************************************************************************************
GBSARMS_STATA
general adaptive rejection with metropolis step for STATA
*/
void gbsarms_stata(
string scalar cmd,
string scalar    namelogp,
real scalar      logp,
string scalar    b,
real scalar      p,
string scalar    g,
real scalar      minx,
real scalar      maxx,
| real scalar      maxtry)
{
   t = st_matrix(b)
   gbsarms_generic("s",cmd,namelogp,logp,b,t,p,g,maxtry,minx,maxx,trys=0,fail=0)
   st_matrix(b,t)
   st_local("try",strofreal(trys))
   st_local("fail",strofreal(fail))
   st_local("logp",strofreal(logp))
}
mata mlib add libmcmc gbsarms_stata()
/************************************************************************************
GBSARMS_GENERIC
general adaptive rejection with metropolis step for STATA or MATA
*/
void gbsarms_generic(
string scalar source,
string scalar cmd,
string scalar    namelogp,
real scalar      logp,
string scalar    nametheta,
real rowvector   theta,
real scalar      p,
string scalar    g,
real scalar      maxtry,
real scalar      minx,
real scalar      maxx,
| real scalar      trys,
real scalar      fail,
pointer(function) userprog,
real matrix      X)
{
   real scalar j, k, step, maxy, try1, oldlogp, oldx
   real rowvector grid, y, a, b, mid, ymid, split, line, area, areaLeft, areaRight
   /*---------------------------------
   calculate current point if necessary
   ---------------------------------*/
   oldx = theta[p]
   if ( logp == . ) {
      if ( source == "m" ) oldlogp = (*userprog)(X,theta,p)
      else {
         st_matrix(nametheta,theta)
         stata(cmd,1)
         oldlogp = st_numscalar(namelogp)
      }
      if ( oldlogp == . ) {
         printf("gbsarms: Unable to calculate log-posterior for parameter %4.0f\n",p)
         matrixprint(" %10.4f",theta)
         exit(1400)
      }
   }
   else oldlogp = logp
   /*-----------------------------------
   calculate over a given grid
   or Create a armsgrid from the end points
   -----------------------------------*/
   glist = tokens(g)
   npts = length(glist)
   grid = J(1,npts,0)
   f = J(1,npts,0)
   for (j=1;j<=npts;j++) {
      grid[j] = strtoreal(glist[j])
   }
   y = J(1,npts,0)
   maxy = 0
      for (j=1;j<=npts;j++) {
         theta[p] = grid[j]
         if ( source == "m" ) y[j] = (*userprog)(X,theta,p)
         else {
            st_matrix(nametheta,theta)
            stata(cmd,1)
            y[j] = st_numscalar(namelogp)
         }
         if ( y[j] == . ) {
            printf("gbsarms: Unable to calculate log-posterior for parameter %4.0f\n",p)
            matrixprint(" %10.4f",theta)
            exit(1400)
         }
         if ( j == 1 | y[j] > maxy ) maxy = y[j]
      }
   for (j=1;j<=npts;j++) {
      y[j] = y[j] - maxy
   }
   /*---------------------------------
   Random point from the Envelope
   ---------------------------------*/
   try1 = 0
   finished = 0
   while( try1 < maxtry & finished == 0 ) {
      try1++
      /*---------------------------------
      Lines to form the envelope
      ---------------------------------*/
      nline = npts - 1
      nregion = npts - 1
	  gmin = 2*grid[1] - grid[npts]
	  if ( minx != . & gmin <= minx ) gmin = (minx+grid[1])/2
	  gmax = 2*grid[npts] - grid[1]
	  if ( maxx != . & gmax >= maxx ) gmax = (maxx+grid[npts])/2
      a = J(1,npts,0)
      b = J(1,npts,0)
      for (j=1;j<=nline;j++) {
         b[j] = (y[j+1]-y[j])/(grid[j+1]-grid[j])
         a[j] =  y[j] - b[j]*grid[j]
      }
      /*---------------------------------
      Area of regions under the envelope
      envelope = max( yH , min(yL,yR) )
      ---------------------------------*/
      totalarea = 0
      mid = J(1,nregion,0)
      ymid = J(1,nregion,0)
      split = J(1,nregion,0)
      area = J(1,nregion,0)
      line = J(1,nregion,0)
      areaLeft = J(1,nregion,0)
      areaRight = J(1,nregion,0)
      totalarea = 0
      for (region=2;region<nregion;region++) {
         mid[region] = (a[region+1]-a[region-1])/(b[region-1]-b[region+1])
         ymid[region] = a[region+1]+b[region+1]*mid[region]
         if ( mid[region] > grid[region] & mid[region] < grid[region+1] ) {
            yH =  a[region]  + b[region]*mid[region]
            if ( ymid[region] > yH ) {
               split[region] = 1
               areaLeft[region] = gbs_area(grid[region],mid[region],a[region-1],b[region-1])
               areaRight[region] = gbs_area(mid[region],grid[region+1],a[region+1],b[region+1])
               area[region] = areaLeft[region]+areaRight[region]
            }
            else {
               split[region] = 0
               line[region] = region
               area[region] = gbs_area(grid[region],grid[region+1],a[region],b[region])
            }
         }
         else {
            split[region] = 0
            mid[region] = (grid[region]+grid[region+1])/2
            yL = a[region-1]+b[region-1]*mid[region]
            yH = a[region]  +b[region]  *mid[region]
            yR = a[region+1]+b[region+1]*mid[region]
            if ( yL > yH & yL < yR ) {
               area[region] = gbs_area(grid[region],grid[region+1],a[region-1],b[region-1])
               line[region] = region - 1
            }
            else if ( yR > yH & yR < yL ) {
               area[region] = gbs_area(grid[region],grid[region+1],a[region+1],b[region+1])
               line[region] = region + 1
            }
            else {
               area[region] = gbs_area(grid[region],grid[region+1],a[region],b[region])
               line[region] = region
            }
         }
         totalarea = totalarea + area[region]
      }
      split[1] = 0
      mid[1] = (gmin+grid[2])/2
      yH = a[1]+b[1]*mid[1]
      yR = a[2]+b[2]*mid[1]
      if ( yH < yR ) {
         area[1] = gbs_area(gmin,grid[2],a[2],b[2])
         line[1] = 2
      }
      else {
         area[1] = gbs_area(gmin,grid[2],a[1],b[1])
         line[1] = 1
      }
      split[nregion] = 0
      mid[nregion] = (grid[npts-1]+gmax)/2
      yL = a[nregion-1]+b[nregion-1]*mid[nregion]
      yH = a[nregion]+b[nregion]*mid[nregion]
      if ( yH < yL ) {
         area[nregion] = gbs_area(grid[npts-1],gmax,a[nregion-1],b[nregion-1])
         line[nregion] = nregion-1
      }
      else {
         area[nregion] = gbs_area(grid[npts-1],gmax,a[nregion],b[nregion])
         line[nregion] = nregion
      }
      totalarea = totalarea + area[1] + area[nregion]
      /*---------------------------------
      Select a random block under envelope
      ---------------------------------*/
      u = totalarea*runiform(1,1)
      block = 1
      sum = area[1]
      while( u > sum ) {
         block++
         sum = sum + area[block]
      }
      u = u - sum + area[block]
	  if ( block == 1 ) {
	     xL = gmin
		 xR = grid[2]
		 j = line[1]
	  }
	  else if ( block == nregion ) {
	     xL = grid[nregion-1]
		 xR = gmax
		 j = line[nregion]
	  }
	  else {
         xL = grid[block]
         xR = grid[block+1]
         if ( split[block] == 1 ) {
           if ( u < areaLeft[block] ) {
            j = block - 1
            xR = mid[block]
           }
           else {
            xL = mid[block]
            j = block + 1
            u = u - areaLeft[block]
           }
		 }
         else j = line[block]
      }
      /*---------------------------------
      Random point in that block
      ---------------------------------*/
      x = (log(b[j]*u+exp(a[j]+b[j]*xL))-a[j])/b[j]
      if ( x < xL | x > xR ) x = xL + u/exp(a[j])
      yh = a[j] + b[j]*x
      if ( hasmissing(x) ) {
         printf("gbsarms: Unable to generate random point for parameter %4.0f with grid\n",p)
         for (j=1;j<=npts;j++) {
            printf("%4.0f: %10.5f %10.5f\n", j,grid[j],f[j])
         }
         exit(1400)
      }
      theta[p] = x
      if ( source == "m" ) logp = (*userprog)(X,theta,p)
      else {
         st_matrix(nametheta,theta)
         stata(cmd,1)
         logp = st_numscalar(namelogp)
      }
      if ( logp == . ) {
         printf("gbsarms: Unable to calculate log-posterior for parameter %4.0f\n",p)
         matrixprint(" %10.4f",theta)
         exit(1400)
      }
	  if ( oldx <= grid[1] ) block = 1
	  else if ( oldx > grid[nregion] ) block = nregion
	  else {
         block = 1
         while( grid[block] < oldx ) block++
         block = block - 1
      }
      if ( split[block] == 0 ) k = line[block]
      else if ( oldx < mid[block] ) k = block - 1
      else k = block + 1
      oldh = a[k] + b[k]*oldx
      /*---------------------------------
      ARS Rejection Step
      ---------------------------------*/
      if ( log(runiform(1,1)) < (logp-maxy-yh) ) {
         /*---------------------------------
         Metropolis Step
         ---------------------------------*/
         finished = 1
         minold = oldlogp - maxy
         if ( oldh < minold ) minold = oldh
         minnew = logp - maxy
         if ( yh < minnew) minnew = yh
         if ( log(runiform(1,1)) > (logp + minold - oldlogp - minnew) ) {
            /*-----------------------
            Fails Metropolis test
            -----------------------*/
            logp = oldlogp
            theta[p] = oldx
         }
      }
      else {
         /*---------------------------------
         ARS Rejection so update grid
         ---------------------------------*/
         update = 1
         step = 0.01*(grid[npts]-grid[1])
         for (i=1;i<=npts;i++) {
            if ( abs(grid[i]-x) < step ) update = 0
         }
         if ( update == 1 ) {
		    npts++
            newgrid = J(1,npts,0)
            newys = J(1,npts,0)
            i = 1
			for (j=1;j<npts;j++) {
               if ( grid[i] < x ) {
               newgrid[i] = grid[i]
               newys[i] = y[i]
               i++
			   }
            }
            newgrid[i] = x
            newys[i] = logp - maxy
            j = i + 1
            while( j <= npts ) {
               newgrid[j] = grid[i]
               newys[j] = y[i]
               j++
               i++
            }
            grid = J(1,npts,0)
            grid = newgrid
            y = J(1,npts,0)
            y = newys
            /*------------------
            Re-scale
            ------------------*/
            nmaxy = y[1]
            for (j=2;j<npts;j++) {
               if ( y[j] > nmaxy ) nmaxy = y[j]
            }
            for (j=1;j<=npts;j++) {
               y[j] = y[j] - nmaxy
            }
            maxy = maxy + nmaxy
         }
      }
   }
   /*--------------------------------------
   Unsuccessful so go back to the old point
   --------------------------------------*/
   if ( finished == 0 ) {
      logp = oldlogp
      theta[p] = oldx
   }
   if ( args() > 7 ) trys = try1
   if ( args() > 8 ) fail = 1 - finished
}
mata mlib add libmcmc gbsarms_generic()

/************************************************************************************
GBSARS
adaptive rejection sampling  for log-concave functions
*/
void gbsars(
real scalar      logp,
pointer(function) userprog,
real matrix      X,
real rowvector   theta,
real scalar      p,
string scalar    g,
real scalar      maxtry,
real scalar      minx,
real scalar      maxx,
| real scalar      trys,
real scalar      fail)
{
   gbsars_generic("m","","",logp,"",theta,p,g,maxtry,minx,maxx,trys,fail,userprog,X)
}
mata mlib add libmcmc gbsars()
/************************************************************************************
GBSARS_STATA
adaptive rejection sampling  for log-concave functions - called from Stata
*/
void gbsars_stata(
string scalar cmd,
string scalar    namelogp,
real scalar      logp,
string scalar    b,
real scalar      p,
string scalar    g,
real scalar      minx,
real scalar      maxx,
| real scalar      maxtry)
{
   t = st_matrix(b)
   gbsars_generic("s",cmd,namelogp,logp,b,t,p,g,maxtry,minx,maxx,trys=0,fail=0)
   st_matrix(b,t)
   st_local("try",strofreal(trys))
   st_local("fail",strofreal(fail))
   st_local("logp",strofreal(logp))
}
mata mlib add libmcmc gbsars_stata()
/************************************************************************************
gbsars_GENERIC
adaptive rejection sampling  for log-concave functions - called from Stata or Mata
*/
void gbsars_generic(
string scalar source,
string scalar cmd,
string scalar    namelogp,
real scalar      logp,
string scalar    nametheta,
real rowvector   theta,
real scalar      p,
string scalar    g,
real scalar      maxtry,
real scalar      minx,
real scalar      maxx,
| real scalar    trys,
real scalar      fail,
pointer(function) userprog,
real matrix      X)
{
   real scalar j, step, maxy, x
   real rowvector grid, y, f, a, b, mid, ymid, split, line, area, areaLeft, areaRight

   oldx = theta[p]
   /*-----------------------------------
   calculate over a given grid
   or Create a grid from the end points
   -----------------------------------*/
   glist = tokens(g)
   npts = length(glist)
   grid = J(1,npts,0)
   f = J(1,npts,0)
   for (j=1;j<=npts;j++) {
      grid[j] = strtoreal(glist[j])
   }
   y = J(1,npts,0)
   maxy = 0
      for (j=1;j<=npts;j++) {
         theta[p] = grid[j]
         if ( source == "m" ) y[j] = (*userprog)(X,theta,p)
         else {
            st_matrix(nametheta,theta)
            stata(cmd,1)
            y[j] = st_numscalar(namelogp)
         }
         if ( y[j] == . ) {
            printf("gbsars: Unable to calculate log-posterior for parameter %4.0f\n",p)
            matrixprint(" %10.4f",theta)
            exit(1400)
         }
         if ( j == 1 | y[j] > maxy ) maxy = y[j]
      }
   for (j=1;j<=npts;j++) {
      y[j] = y[j] - maxy
   }
   /*---------------------------------
   Random point from the Envelope
   ---------------------------------*/
   try1 = 0
   finished = 0
   while( try1 < maxtry & finished == 0 ) {
      try1++
      /*---------------------------------
      Lines to form the envelope
      ---------------------------------*/
      nline = npts - 1
      nregion = npts - 1
	  gmin = 2*grid[1] - grid[npts]
	  if ( minx != . & gmin <= minx ) gmin = (minx+grid[1])/2
	  gmax = 2*grid[npts] - grid[1]
	  if ( maxx != . & gmax >= maxx ) gmax = (maxx+grid[npts])/2
      a = J(1,npts,0)
      b = J(1,npts,0)
      for (j=1;j<=nline;j++) {
         b[j] = (y[j+1]-y[j])/(grid[j+1]-grid[j])
         a[j] =  y[j] - b[j]*grid[j]
      }
      /*---------------------------------
      Area of regions under the envelope
      ---------------------------------*/
      totalarea = 0
      mid = J(1,nregion,0)
      ymid = J(1,nregion,0)
      area = J(1,nregion,0)
      areaLeft = J(1,nregion,0)
      areaRight = J(1,nregion,0)
      totalarea = 0
      for (region=2;region<nregion;region++) {
         mid[region] = (a[region+1]-a[region-1])/(b[region-1]-b[region+1])
         ymid[region] = a[region+1]+b[region+1]*mid[region]
         if ( mid[region] > grid[region] & mid[region] < grid[region+1] ) {
            areaLeft[region] = gbs_area(grid[region],mid[region],a[region-1],b[region-1])
            areaRight[region] = gbs_area(mid[region],grid[region+1],a[region+1],b[region+1])
            area[region] = areaLeft[region]+areaRight[region]
         }
         else {
            printf("gbsars: Posterior not log-concave for parameter %4.0f\n",p)
            printf("in (%10.4f,%10.4f)\n",grid[region],grid[region+1])
			printf("Left  line %10.4f %10.4f\n",a[region-1],b[region-1])
			printf("Right line %10.4f %10.4f\n",a[region+1],b[region+1])
			printf("Current Grid\n")
            for (j=1;j<=npts;j++) {
               printf("%4.0f:  %10.4f  %10.4f\n",j,grid[j],y[j])
            }
            theta[p] = oldx
            if ( args() > 7 ) trys = try1
            if ( args() > 8 ) fail = 1
            exit(499)
         }
         totalarea = totalarea + area[region]
      }
      
      area[1] = gbs_area(gmin,grid[2],a[2],b[2])
      area[nregion] = gbs_area(grid[npts-1],gmax,a[nregion-1],b[nregion-1])
      totalarea = totalarea + area[1] + area[nregion]
      /*---------------------------------
      Select a random block under envelope
      ---------------------------------*/
      u = totalarea*runiform(1,1)
      block = 1
      sum = area[1]
      while( u > sum ) {
         block++
         sum = sum + area[block]
      }
      u = u - sum + area[block]
      if ( block == 1 ) {
	     xL = gmin
		 j = 2
	  }
	  else if ( block == nregion ) {
	     xL = grid[block] 
		 j = nregion-1
	  }
      else {
         if ( u < areaLeft[block] ) {
		    xL = grid[block]
		    j = block - 1
		 }	
         else {
            xL = mid[block]
            j = block + 1
            u = u - areaLeft[block]
         }
      }
      /*---------------------------------
      Random point in that block
      ---------------------------------*/
      if ( abs(b[j]) > 0.00001 ) x = (log(b[j]*u+exp(a[j]+b[j]*xL))-a[j])/b[j]
      else x = xL + u/exp(a[j])	  
      yh = a[j] + b[j]*x
      theta[p] = x
      if ( x == . ) {
         printf("gbsars: Unable to generate random point for parameter %4.0f with grid\n",p)
         for (j=1;j<=npts;j++) {
            printf("%4.0f: %10.5f %10.5f\n", j,grid[j],f[j])
         }
         exit(1400)
      }
      if ( source == "m" ) logp = (*userprog)(X,theta,p)
      else {
         st_matrix(nametheta,theta)
         stata(cmd,1)
         logp = st_numscalar(namelogp)
      }
      if ( logp == . ) {
         printf("gbsars: Unable to calculate log-posterior for parameter %4.0f\n",p)
         matrixprint(" %10.4f",theta)
         exit(1400)
      }
      /*---------------------------------
      ARS Rejection Step
      ---------------------------------*/
      u = log(runiform(1,1))
         /*---------------------------------
         ARS Accept point
         ---------------------------------*/
      if ( u < (logp-maxy-yh) ) {
         finished = 1
      }
         /*---------------------------------
         ARS Rejection: update grid
         ---------------------------------*/
      else {
         /*---------------------------------
         ARS Rejection: update grid
         ---------------------------------*/
         update = 1
         step = 0.01*(grid[npts]-grid[1])
         for (i=1;i<=npts;i++) {
            if ( abs(grid[i]-x) < step ) update = 0
         }
         if ( update == 1 ) {
		    npts++
            newgrid = J(1,npts,0)
            newys = J(1,npts,0)
            i = 1
			for (j=1;j<npts;j++) {
               if ( grid[i] < x ) {
                  newgrid[i] = grid[i]
                  newys[i] = y[i]
                  i++
			   }
            }
            newgrid[i] = x
            newys[i] = logp-maxy
            j = i + 1
            while( j <= npts ) {
               newgrid[j] = grid[i]
               newys[j] = y[i]
               j++
               i++
            }
            grid = J(1,npts,0)
            grid = newgrid
            y = J(1,npts,0)
            y = newys
            /*------------------
            Re-scale
            ------------------*/
            nmaxy = y[1]
            gap = 0
            for (j=2;j<=npts;j++) {
               if ( y[j] > nmaxy ) nmaxy = y[j]
            }
            for (j=1;j<=npts;j++) {
               y[j] = y[j] - nmaxy
            }
            maxy = maxy + nmaxy
         }
      }
   }
   /*--------------------------------------
   Unsuccessful so go back to the old point
   --------------------------------------*/
   if ( finished == 0 ) {
      logp = oldlogp
      theta[p] = oldx
   }
   if ( args() > 7 ) trys = try1
   if ( args() > 8 ) fail = 1 - finished
}
mata mlib add libmcmc gbsars_generic()
/************************************************************************************
GBS_AREA
area under a piecewise exponential
*/
real scalar gbs_area(
real scalar x1,
real scalar x2,
real scalar a,
real scalar b)
{
   if ( abs(b) > 0.0001 ) return(exp(a+b*x2)/b - exp(a+b*x1)/b)
   else return((x2-x1)*exp(a))
}
mata mlib add libmcmc gbs_area()
/************************************************************************************
LOGDENSITY
LOG DENSITIES OF STANDARD DISTRIBUTIONS
parameters may be scalars or vectors
optional frequency weight can be set to zero to restrict observations
constants dropped from the calculations
Number of parameters required depends on the distibution
no checks of comparability of sizes because on need for speed
************************************************************************************/
real scalar logdensity(
string scalar distribution,
real matrix y,
real matrix p1,
| real matrix p2,
real matrix p3,
real matrix p4)
{
   if ( distribution == "bernoulli" ) {
      if ( args() < 3 || args() > 4 ) exit(3000)
      if ( args() > 3 ) w = p2; else w = 1
      return( sum(w:*(y:*log(p1):+(1:-y):*log(1:-p1)),1) )
   }
   else if ( distribution == "beta" ) {
      if ( args() < 4 || args() > 5 ) exit(3000)
      if ( args() == 5 ) w = p3; else w = 1
      return( sum(w:*((p1:-1):*log(y):+(p2:-1):*log(1:-y):+lngamma(p1:+p2):-lngamma(p1):-lngamma(p2)),1) )
   }
   else if ( distribution == "binomial" ) {
      if ( args() < 4 || args() > 5 ) exit(3000)
      if ( args() == 5 ) w = p3; else w = 1
      return( sum(w:*(y:*log(p2):+(p1:-y):*log(1:-p2)),1) )
   }
   else if ( distribution == "categorical" ) {
      if ( args() < 3 || args() > 4 ) exit(3000)
      if ( args() == 4 ) w = p2; else w = 1
      return( sum(w:*( log(p1[y]) ),1) )
   }
   else if ( distribution == "chisqr" ) {
      if ( args() < 3 || args() > 4 ) exit(3000)
      if ( args() == 4 ) w = p2; else w = 1
      return( sum(w:*(-p1*0.34657359:+(p1:/2:-1):*log(y):-y:/2:-lngamma(p1/2)),1) )
   }
   else if ( distribution == "dirichlet" ) {
      if ( args() < 3 || args() > 4 ) exit(3000)
      if ( args() == 4 ) w = p2; else w = 1
      la = lngamma(sum(p1:*w))
      return( sum(w*((p1:-1):*log(y):-lngamma(p1)),1 ) + la )
   }
   else if ( distribution == "exponential" ) {
      if ( args() < 3 || args() > 4 ) exit(3000)
      if ( args() == 4 ) w = p2; else w = 1
      return( sum(w:*( -log(p1):-y:/p1 ),1) )
   }
   else if ( distribution == "gamma" ) {
      if ( args() < 4 || args() > 5 ) exit(3000)
      if ( args() == 5 ) w = p3; else w = 1
      return( sum(w:*((p1:-1):*log(y):-y:/p2:-p1:*log(p2):-lngamma(p1)),1) )
   }
   else if ( distribution == "gengamma" ) {
      if ( args() < 5 || args() > 6 ) exit(3000)
      if ( args() == 6 ) w = p4; else w = 1
      return( sum(w:*( log(p3:/p2):+(p1:-1):*log(y:/p2):-(y:/p2):^p3:-lngamma(p1:/p3) ),1) )
   }
   else if ( distribution == "igamma" ) {
      if ( args() < 4 || args() > 5 ) exit(3000)
      if ( args() == 5 ) w = p3; else w = 1
      return( sum(w:*( p1:*log(p2):-(p1:+1):*log(y):-(p2:/y):-lngamma(p1) ),1) )
   }
   else if ( distribution == "igauss" ) {
      if ( args() < 4 || args() > 5 ) exit(3000)
      if ( args() == 5 ) w = p3; else w = 1
      return( sum(w:*( 0.5:*log(p2:/(y:*y:*y))-0.5:*p2:*(y:-p1):*(y:-p1):/(y:*p1:*p1) ),1) )
   }
   else if ( distribution == "laplace" ) {
      if ( args() < 4 || args() > 5 ) exit(3000)
      if ( args() == 5 ) w = p3; else w = 1
      return( sum(w:*( -log(2:*p2):-abs(y:-p1):/p2 ),1) )
   }
   else if ( distribution == "logistic" ) {
      if ( args() < 4 || args() > 5 ) exit(3000)
      if ( args() == 5 ) w = p3; else w = 1
      return( sum(w:*( -(y:-p1):/p2 :- log(p2) :- 2:*log(1:+exp(-(y:-p1):/p2)) ),1) )
   }
   else if ( distribution == "lognormal" ) {
      if ( args() < 4 || args() > 5 ) exit(3000)
      if ( args() == 5 ) w = p3; else w = 1
      ly = log(y)
      return( sum(w:*( -log(p2):-0.5:*(ly:-p1):*(ly:-p1):/(p2:*p2):-y ),1) )
   }
   else if ( distribution == "multinomial" ) {
      if ( args() != 3 ) exit(3000)
      return( y:*log(p1) )
   }
   else if ( distribution == "mnormal" ) {
      if ( args() != 4 ) exit(3000)
      return( -0.5*log(det(p2)) - 0.5*(y-p1)'*invsym(p2)*(y-p1) )
   }
   else if ( distribution == "mt" ) {
      if ( args() != 5 ) exit(3000)
      p = rows(p2)
      return( -0.5*log(abs(det(p2)))-0.5*(p3+p)*log(1+sum((y-p1)'*invsym(p2)*(y-p1),1)/p3)+lngamma((p+p3)/2)-lngamma(p3/2) )
   }
   else if ( distribution == "negbin" ) {
      if ( args() < 4 || args() > 5 ) exit(3000)
      if ( args() == 5 ) w = p3; else w = 1
      return( sum(w:*( p2:*log(p1):+y:*log(1:-p1):+lngamma(y:+p2):-lngamma(p2):-lngamma(y:+1) ),1) )
   }
   else if ( distribution == "normal" ) {
      if ( args() < 4 || args() > 5 ) exit(3000)
      if ( args() == 5 ) w = p3; else w = 1
      return( sum(w:*(-log(p2):-0.5:*(y:-p1):*(y:-p1):/(p2:*p2)),1) )
   }
   else if ( distribution == "pareto" ) {
      if ( args() < 4 || args() > 5 ) exit(3000)
      if ( args() == 5 ) w = p3; else w = 1
      return( sum(w:*( log(p1):+p1:*log(p2):-(p1:+1):*log(y) ),1) )
   }
   else if ( distribution == "poisson" ) {
      if ( args() < 3 || args() > 4 ) exit(3000)
      if ( args() > 3 ) w = p2; else w = 1
      return( sum(w:*(-p1:+y:*log(p1)),1) )
   }
   else if ( distribution == "t" ) {
      if ( args() < 5 || args() > 6 ) exit(3000)
      if ( args() == 6 ) w = p4; else w = 1
      return( sum(w:*( -log(p2):-((p3:+1):/2):*log(1:+(y:-p1):*(y:-p1):/(p2:*p2:*p3)) ),1) )
   }
   else if ( distribution == "uniform" ) {
      if ( args() < 4 || args() > 5 ) exit(3000)
      if ( args() == 5 ) w = p3; else w = 1
      r = (y:>=p1):*(y:<=p2)
      return( sum(w:*( -log(p2:-p1):/r ),1) )
   }
   else if ( distribution == "weibull" ) {
      if ( args() < 4 || args() > 5 ) exit(3000)
      if ( args() == 5 ) w = p3; else w = 1
      return( sum(w:*( log(p1:/p2):+(p1:-1):*log(y:/p2):-(y:/p2):^(p1) ),1) )
   }
   else if ( distribution == "wishart" ) {
      if ( args() != 4 ) exit(3000)
      p = rows(p1)
      return( -0.5*trace(invsym(p1)*y)+0.5*(p2-p-1)*log(det(y))-0.5*p2*log(det(p1))  )
   }
   else {
      printf("Distribution %10s not recognised\n",distribution)
	  exit(3000)
   }
}
mata mlib add libmcmc logdensity()

/************************************************************************************
Print a matrix
*/
void matrixprint(string scalar fmt,real matrix t, | string scalar title)
{
   real scalar R,C
   R=rows(t)
   C=cols(t)
   if ( args() == 3 ) printf("%s\n",title)
   for (r=1;r<=R;r++) {
      for (c=1;c<=C;c++) {
         printf(fmt,t[r,c])
         if ( c == C | mod(c,5) == 0 ) printf("\n")
         else printf(" ")
      }
   }
}
mata mlib add libmcmc matrixprint()
mata mlib index
end
